import sys
from nose.plugins import Plugin
from subprocess import Popen


class WatchPlugin(Plugin):
    """
    Plugin that use watchdog for continuos tests run.
    """
    name = 'watch'
    is_watching = False
    sys = sys

    def call(self, args):
        Popen(args).wait()

    def finalize(self, result):
        argv = list(self.sys.argv)
        argv.remove('--with-watch')
        watchcmd = 'clear && ' + ' '.join(argv)
        call_args = ['watchmedo', 'shell-command', '-c',
            watchcmd, '-R', '-p', '*.py', '.']
        try:
            self.call(call_args)
        except KeyboardInterrupt:
            self.sys.stdout.write('\nStopped\n')




"""
Main plugin tests.

.. note:: Please to not run nosetests with this plugin for testing the plugin
   itself. Also, mock.patch is not much help here as it would try to mock the
   module that would be already imported by nose itself if plugin was installed
   globally (it would take module from sys.modules).
"""
import unittest
from mock import Mock
from ..plugin import WatchPlugin


class TestWatchPlugin(unittest.TestCase):

    def setUp(self):
        self.plugin = WatchPlugin()
        self.plugin.sys = Mock()
        self.plugin.sys.argv = ['program', 'arg1', '--with-watch', 'arg3', 'arg4']

    def test_finalize(self):
        self.plugin.call = Mock()
        self.plugin.finalize(Mock())

        watchcmd = 'clear && program arg1 arg3 arg4'
        self.plugin.call.assert_called_once_with([
            'watchmedo', 'shell-command', '-c', watchcmd,
            '-R', '-p', '*.py', '.'])

    def test_finalize_interrupted(self):
        self.plugin.call = Mock(side_effect=KeyboardInterrupt)
        self.plugin.finalize(Mock())
        self.plugin.sys.stdout.write.assert_called_once_with('\nStopped\n')




"""
A nose plugin that re-runs test suite on filesystem event.
"""


VERSION = (0, 9, 1, 'dev')

__version__ = '.'.join((str(each) for each in VERSION[:4]))

def get_version():
    """
    Returns shorter version (digit parts only) as string.
    """
    return '.'.join((str(each) for each in VERSION[:4]))




import nosewatch
from setuptools import setup, find_packages


setup(
    name='nose-watch',
    version=nosewatch.get_version(),
    author='Lukasz Balcerzak',
    author_email='lukaszbalcerzak@gmail.com',
    description=nosewatch.__doc__,
    url='https://github.com/lukaszb/nose-watch',
    packages=find_packages(),
    long_description=open('README.rst').read(),
    setup_requires=['nose>=1.0'],
    install_requires=['watchdog>=0.6'],
    tests_require=['mock'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Testing',
    ],
    entry_points={
        'nose.plugins.0.10': [
            'watch = nosewatch.plugin:WatchPlugin'
        ]
    }
)

